@@ -9,10 +9,13 @@ from contextlib import suppress, contextmanager
 from copy import copy
 from typing import Callable, Mapping, Optional, Union
 from ssl import SSLContext
+import abc
 
 from werkzeug.http import parse_authorization_header
 from werkzeug.serving import make_server
 from werkzeug.wrappers import Request, Response
+import werkzeug.urls
+from werkzeug.datastructures import MultiDict
 
 URI_DEFAULT = ""
 METHOD_ALL = "__ALL"
@@ -123,6 +126,106 @@ HeaderValueMatcher.DEFAULT_MATCHERS = defaultdict(
 )
 
 
+class QueryMatcher(abc.ABC):
+    """
+    Abstract class for QueryMatchers
+
+    get_comparing_values should return a 2-element tuple whose elements will
+    be compared.
+    """
+    def match(self, request_query_string: bytes) -> bool:
+        values = self.get_comparing_values(request_query_string)
+        return values[0] == values[1]
+
+    @abc.abstractmethod
+    def get_comparing_values(self, request_query_string: bytes) -> tuple:
+        pass
+
+class StringQueryMatcher(QueryMatcher):
+    """
+    Matches a query for a string or bytes specified
+    """
+    def __init__(self, query_string: Union[bytes, str]):
+        """
+        :param query_string: the query string will be compared to this string or bytes.
+            If string is specified, it will be encoded by the encode() method.
+            The query must not start with '?' but will be exactly (byte-by-byte) equal
+            the actual query string of the incoming request.
+        """
+        if query_string is not None and not isinstance(query_string, (str, bytes)):
+            raise TypeError("query_string must be a string, or a bytes-like object")
+
+        self.query_string = query_string
+
+    def get_comparing_values(self, request_query_string: bytes) -> tuple:
+        if self.query_string is not None:
+            if isinstance(self.query_string, str):
+                query_string = self.query_string.encode()
+            elif isinstance(self.query_string, bytes):
+                query_string = self.query_string
+            else:
+                raise TypeError("query_string must be a string, or a bytes-like object")
+
+        return (request_query_string, query_string)
+
+
+class MappingQueryMatcher(QueryMatcher):
+    """
+    Matches a query string to a dictionary or MultiDict specified
+    """
+
+    def __init__(self, query_dict: [Mapping, MultiDict]):
+        """
+        :param query_dict: if dictionary (Mapping) is specified, it will be used as a
+            key-value mapping where both key and value should be string. If there are multiple
+            values specified for the same key in the request, the first element will be used.
+            If you want to match multiple values, use a MultiDict object from werkzeug, which
+            represents multiple values for one key.
+        """
+        self.query_dict = query_dict
+
+    def get_comparing_values(self, request_query_string: bytes) -> tuple:
+        query = werkzeug.urls.url_decode(request_query_string)
+        if isinstance(self.query_dict, MultiDict):
+            return (query, self.query_dict)
+        else:
+            return (query.to_dict(), dict(self.query_dict))
+
+
+class BooleanQueryMatcher(QueryMatcher):
+    """
+    Matches the query depending on the boolean value
+    """
+    def __init__(self, result: bool):
+        """
+        :param result: if this parameter is true, the query match will be always
+            successful. Otherwise, no query match will be successful.
+        """
+        self.result = result
+
+    def get_comparing_values(self, request_query_string):
+        if self.result:
+            return (True, True)
+        else:
+            return (True, False)
+
+
+def _create_query_matcher(query_string: Union[None, QueryMatcher, str, bytes, Mapping]) -> QueryMatcher:
+    if isinstance(query_string, QueryMatcher):
+        return query_string
+
+    if query_string is None:
+        return BooleanQueryMatcher(True)
+
+    if isinstance(query_string, (str, bytes)):
+        return StringQueryMatcher(query_string)
+
+    if isinstance(query_string, Mapping):
+        return MappingQueryMatcher(query_string)
+
+    raise TypeError("Unable to cast this type to QueryMatcher: {!r}".format(type(query_string)))
+
+
 class RequestMatcher:
     """
     Matcher object for the incoming request.
@@ -136,7 +239,12 @@ class RequestMatcher:
         by default, see `data_encoding`) or a bytes object.
     :param data_encoding: the encoding used for data parameter if data is a string.
     :param headers: dictionary of the headers of the request to be matched
-    :param query_string: the http query string starting with ``?``, such as ``?username=user``
+    :param query_string: the http query string, after ``?``, such as ``username=user``.
+        If string is specified it will be encoded to bytes with the encode method of
+        the string. If dict is specified, it will be matched to the ``key=value`` pairs
+        specified in the request. If multiple values specified for a given key, the first
+        value will be used. If multiple values needed to be handled, use ``MultiDict``
+        object from werkzeug.
     """
 
     def __init__(
@@ -146,12 +254,13 @@ class RequestMatcher:
             data: Union[str, bytes, None] = None,
             data_encoding: str = "utf-8",
             headers: Optional[Mapping[str, str]] = None,
-            query_string: Optional[str] = None,
+            query_string: Union[None, QueryMatcher, str, bytes, Mapping] = None,
             header_value_matcher: Optional[HeaderValueMatcher] = None):
 
         self.uri = uri
         self.method = method
         self.query_string = query_string
+        self.query_matcher = _create_query_matcher(self.query_string)
 
         if headers is None:
             self.headers = {}
@@ -206,7 +315,7 @@ class RequestMatcher:
         if self.method != METHOD_ALL and self.method != request.method:
             retval.append(("method", request.method, self.method))
 
-        if self.query_string is not None and self.query_string != request.query_string:
+        if not self.query_matcher.match(request.query_string):
             retval.append(("query_string", request.query_string, self.query_string))
 
         request_headers = {}
@@ -463,7 +572,7 @@ class HTTPServer:   # pylint: disable=too-many-instance-attributes
             data: Union[str, bytes, None] = None,
             data_encoding: str = "utf-8",
             headers: Optional[Mapping[str, str]] = None,
-            query_string: Optional[str] = None,
+            query_string: Union[None, QueryMatcher, str, bytes, Mapping] = None,
             header_value_matcher: Optional[HeaderValueMatcher] = None,
             handler_type: HandlerType = HandlerType.PERMANENT) -> RequestHandler:
         """
@@ -488,7 +597,12 @@ class HTTPServer:   # pylint: disable=too-many-instance-attributes
             by default, see `data_encoding`) or a bytes object.
         :param data_encoding: the encoding used for data parameter if data is a string.
         :param headers: dictionary of the headers of the request to be matched
-        :param query_string: the http query string starting with ``?``, such as ``?username=user``
+        :param query_string: the http query string, after ``?``, such as ``username=user``.
+            If string is specified it will be encoded to bytes with the encode method of
+            the string. If dict is specified, it will be matched to the ``key=value`` pairs
+            specified in the request. If multiple values specified for a given key, the first
+            value will be used. If multiple values needed to be handled, use ``MultiDict``
+            object from werkzeug.
         :param header_value_matcher: :py:class:`HeaderValueMatcher` that matches values of headers.
         :param handler_type: type of handler
 
@@ -520,7 +634,7 @@ class HTTPServer:   # pylint: disable=too-many-instance-attributes
             data: Union[str, bytes, None] = None,
             data_encoding: str = "utf-8",
             headers: Optional[Mapping[str, str]] = None,
-            query_string: Optional[str] = None,
+            query_string: Union[None, QueryMatcher, str, bytes, Mapping] = None,
             header_value_matcher: Optional[HeaderValueMatcher] = None) -> RequestHandler:
         """
         Create and register a oneshot request handler.
@@ -534,7 +648,12 @@ class HTTPServer:   # pylint: disable=too-many-instance-attributes
             by default, see `data_encoding`) or a bytes object.
         :param data_encoding: the encoding used for data parameter if data is a string.
         :param headers: dictionary of the headers of the request to be matched
-        :param query_string: the http query string starting with ``?``, such as ``?username=user``
+        :param query_string: the http query string, after ``?``, such as ``username=user``.
+            If string is specified it will be encoded to bytes with the encode method of
+            the string. If dict is specified, it will be matched to the ``key=value`` pairs
+            specified in the request. If multiple values specified for a given key, the first
+            value will be used. If multiple values needed to be handled, use ``MultiDict``
+            object from werkzeug.
         :param header_value_matcher: :py:class:`HeaderValueMatcher` that matches values of headers.
 
         :return: Created and register :py:class:`RequestHandler`.
@@ -558,7 +677,7 @@ class HTTPServer:   # pylint: disable=too-many-instance-attributes
             data: Union[str, bytes, None] = None,
             data_encoding: str = "utf-8",
             headers: Optional[Mapping[str, str]] = None,
-            query_string: Optional[str] = None,
+            query_string: Union[None, QueryMatcher, str, bytes, Mapping] = None,
             header_value_matcher: Optional[HeaderValueMatcher] = None) -> RequestHandler:
         """
         Create and register a ordered request handler.
@@ -572,7 +691,12 @@ class HTTPServer:   # pylint: disable=too-many-instance-attributes
             by default, see `data_encoding`) or a bytes object.
         :param data_encoding: the encoding used for data parameter if data is a string.
         :param headers: dictionary of the headers of the request to be matched
-        :param query_string: the http query string starting with ``?``, such as ``?username=user``
+        :param query_string: the http query string, after ``?``, such as ``username=user``.
+            If string is specified it will be encoded to bytes with the encode method of
+            the string. If dict is specified, it will be matched to the ``key=value`` pairs
+            specified in the request. If multiple values specified for a given key, the first
+            value will be used. If multiple values needed to be handled, use ``MultiDict``
+            object from werkzeug.
         :param header_value_matcher: :py:class:`HeaderValueMatcher` that matches values of headers.
 
         :return: Created and register :py:class:`RequestHandler`.
